@@ -7,16 +7,7 @@
 package beacon
 
 import (
-	"os"
-	"strings"
-
 	"github.com/syncthing/syncthing/lib/logger"
 )
 
-var (
-	l = logger.DefaultLogger.NewFacility("beacon", "Multicast and broadcast discovery")
-)
-
-func init() {
-	l.SetDebug("beacon", strings.Contains(os.Getenv("STTRACE"), "beacon") || os.Getenv("STTRACE") == "all")
-}
+var l = logger.DefaultLogger.NewFacility("beacon", "Multicast and broadcast discovery")
