@@ -6,6 +6,15 @@
 
 package config
 
+type FolderType int32
+
+const (
+	FolderTypeSendReceive      FolderType = 0
+	FolderTypeSendOnly         FolderType = 1
+	FolderTypeReceiveOnly      FolderType = 2
+	FolderTypeReceiveEncrypted FolderType = 3
+)
+
 func (t FolderType) String() string {
 	switch t {
 	case FolderTypeSendReceive:
@@ -14,6 +23,8 @@ func (t FolderType) String() string {
 		return "sendonly"
 	case FolderTypeReceiveOnly:
 		return "receiveonly"
+	case FolderTypeReceiveEncrypted:
+		return "receiveencrypted"
 	default:
 		return "unknown"
 	}
@@ -31,6 +42,8 @@ func (t *FolderType) UnmarshalText(bs []byte) error {
 		*t = FolderTypeSendOnly
 	case "receiveonly":
 		*t = FolderTypeReceiveOnly
+	case "receiveencrypted":
+		*t = FolderTypeReceiveEncrypted
 	default:
 		*t = FolderTypeSendReceive
 	}
