@@ -6,6 +6,14 @@
 
 package config
 
+type Tuning int32
+
+const (
+	TuningAuto  Tuning = 0
+	TuningSmall Tuning = 1
+	TuningLarge Tuning = 2
+)
+
 func (t Tuning) String() string {
 	switch t {
 	case TuningAuto:
