@@ -38,7 +38,7 @@ func TestIsLANHost(t *testing.T) {
 			AlwaysLocalNets: []string{"10.20.30.0/24"},
 		},
 	}, protocol.LocalDeviceID, events.NoopLogger)
-	s := &service{cfg: cfg}
+	s := &lanChecker{cfg: cfg}
 
 	for _, tc := range cases {
 		res := s.isLANHost(tc.addr)
