@@ -13,13 +13,12 @@ import (
 	"time"
 
 	"github.com/syncthing/syncthing/lib/config"
+	"github.com/syncthing/syncthing/lib/connections/registry"
 	"github.com/syncthing/syncthing/lib/dialer"
 	"github.com/syncthing/syncthing/lib/protocol"
 	"github.com/syncthing/syncthing/lib/relay/client"
 )
 
-const relayPriority = 200
-
 func init() {
 	dialers["relay"] = relayDialerFactory{}
 }
@@ -63,23 +62,25 @@ func (d *relayDialer) Dial(ctx context.Context, id protocol.DeviceID, uri *url.U
 		return internalConn{}, err
 	}
 
-	return newInternalConn(tc, connTypeRelayClient, relayPriority), nil
+	return newInternalConn(tc, connTypeRelayClient, false, d.wanPriority), nil
+}
+
+func (d *relayDialer) Priority(_ string) int {
+	return d.wanPriority
 }
 
 type relayDialerFactory struct{}
 
-func (relayDialerFactory) New(opts config.OptionsConfiguration, tlsCfg *tls.Config) genericDialer {
+func (relayDialerFactory) New(opts config.OptionsConfiguration, tlsCfg *tls.Config, _ *registry.Registry, _ *lanChecker) genericDialer {
 	return &relayDialer{commonDialer{
 		trafficClass:      opts.TrafficClass,
 		reconnectInterval: time.Duration(opts.RelayReconnectIntervalM) * time.Minute,
 		tlsCfg:            tlsCfg,
+		wanPriority:       opts.ConnectionPriorityRelay,
+		lanPriority:       opts.ConnectionPriorityRelay,
 	}}
 }
 
-func (relayDialerFactory) Priority() int {
-	return relayPriority
-}
-
 func (relayDialerFactory) AlwaysWAN() bool {
 	return true
 }
