@@ -9,6 +9,7 @@ package db
 import (
 	"bytes"
 	"context"
+	"crypto/sha256"
 	"encoding/binary"
 	"errors"
 	"fmt"
@@ -18,15 +19,17 @@ import (
 	"time"
 
 	"github.com/greatroar/blobloom"
+	"github.com/thejerf/suture/v4"
+	"google.golang.org/protobuf/proto"
+
+	"github.com/syncthing/syncthing/internal/gen/dbproto"
 	"github.com/syncthing/syncthing/lib/db/backend"
 	"github.com/syncthing/syncthing/lib/events"
 	"github.com/syncthing/syncthing/lib/fs"
 	"github.com/syncthing/syncthing/lib/protocol"
-	"github.com/syncthing/syncthing/lib/sha256"
 	"github.com/syncthing/syncthing/lib/stringutil"
 	"github.com/syncthing/syncthing/lib/svcutil"
 	"github.com/syncthing/syncthing/lib/sync"
-	"github.com/thejerf/suture/v4"
 )
 
 const (
@@ -159,10 +162,6 @@ func (db *Lowlevel) updateRemoteFiles(folder, device []byte, fs []protocol.FileI
 		if err != nil {
 			return err
 		}
-		if ok && unchanged(f, ef) {
-			l.Debugf("not inserting unchanged (remote); folder=%q device=%v %v", folder, devID, f)
-			continue
-		}
 
 		if ok {
 			meta.removeFile(devID, ef)
@@ -216,12 +215,8 @@ func (db *Lowlevel) updateLocalFiles(folder []byte, fs []protocol.FileInfo, meta
 		if err != nil {
 			return err
 		}
-		if ok && unchanged(f, ef) {
-			l.Debugf("not inserting unchanged (local); folder=%q %v", folder, f)
-			continue
-		}
-		blocksHashSame := ok && bytes.Equal(ef.BlocksHash, f.BlocksHash)
 
+		blocksHashSame := ok && bytes.Equal(ef.BlocksHash, f.BlocksHash)
 		if ok {
 			keyBuf, err = db.removeLocalBlockAndSequenceInfo(keyBuf, folder, name, ef, !blocksHashSame, &t)
 			if err != nil {
@@ -529,8 +524,8 @@ func (db *Lowlevel) checkGlobals(folderStr string) (int, error) {
 	var dk []byte
 	ro := t.readOnlyTransaction
 	for dbi.Next() {
-		var vl VersionList
-		if err := vl.Unmarshal(dbi.Value()); err != nil || vl.Empty() {
+		var vl dbproto.VersionList
+		if err := proto.Unmarshal(dbi.Value(), &vl); err != nil || len(vl.Versions) == 0 {
 			if err := t.Delete(dbi.Key()); err != nil && !backend.IsNotFound(err) {
 				return 0, err
 			}
@@ -543,9 +538,9 @@ func (db *Lowlevel) checkGlobals(folderStr string) (int, error) {
 		// we find those and clear them out.
 
 		name := db.keyer.NameFromGlobalVersionKey(dbi.Key())
-		newVL := &VersionList{}
+		newVL := &dbproto.VersionList{}
 		var changed, changedHere bool
-		for _, fv := range vl.RawVersions {
+		for _, fv := range vl.Versions {
 			changedHere, err = checkGlobalsFilterDevices(dk, folder, name, fv.Devices, newVL, ro)
 			if err != nil {
 				return 0, err
@@ -559,7 +554,7 @@ func (db *Lowlevel) checkGlobals(folderStr string) (int, error) {
 			changed = changed || changedHere
 		}
 
-		if newVL.Empty() {
+		if len(newVL.Versions) == 0 {
 			if err := t.Delete(dbi.Key()); err != nil && !backend.IsNotFound(err) {
 				return 0, err
 			}
@@ -580,7 +575,7 @@ func (db *Lowlevel) checkGlobals(folderStr string) (int, error) {
 	return fixed, t.Commit()
 }
 
-func checkGlobalsFilterDevices(dk, folder, name []byte, devices [][]byte, vl *VersionList, t readOnlyTransaction) (bool, error) {
+func checkGlobalsFilterDevices(dk, folder, name []byte, devices [][]byte, vl *dbproto.VersionList, t readOnlyTransaction) (bool, error) {
 	var changed bool
 	var err error
 	for _, device := range devices {
@@ -596,7 +591,7 @@ func checkGlobalsFilterDevices(dk, folder, name []byte, devices [][]byte, vl *Ve
 			changed = true
 			continue
 		}
-		_, _, _, _, _, _, err = vl.update(folder, device, f, t)
+		_, _, _, _, _, _, err = vlUpdate(vl, folder, device, f, t)
 		if err != nil {
 			return false, err
 		}
@@ -618,7 +613,7 @@ func (db *Lowlevel) getIndexID(device, folder []byte) (protocol.IndexID, error)
 
 	var id protocol.IndexID
 	if err := id.Unmarshal(cur); err != nil {
-		return 0, nil
+		return 0, nil //nolint: nilerr
 	}
 
 	return id, nil
@@ -824,11 +819,11 @@ func (db *Lowlevel) gcIndirect(ctx context.Context) (err error) {
 		default:
 		}
 
-		var hashes IndirectionHashesOnly
-		if err := hashes.Unmarshal(it.Value()); err != nil {
+		var hashes dbproto.IndirectionHashesOnly
+		if err := proto.Unmarshal(it.Value(), &hashes); err != nil {
 			return err
 		}
-		db.recordIndirectionHashes(hashes)
+		db.recordIndirectionHashes(&hashes)
 	}
 	it.Release()
 	if err := it.Error(); err != nil {
@@ -932,10 +927,10 @@ func (db *Lowlevel) gcIndirect(ctx context.Context) (err error) {
 }
 
 func (db *Lowlevel) recordIndirectionHashesForFile(f *protocol.FileInfo) {
-	db.recordIndirectionHashes(IndirectionHashesOnly{BlocksHash: f.BlocksHash, VersionHash: f.VersionHash})
+	db.recordIndirectionHashes(&dbproto.IndirectionHashesOnly{BlocksHash: f.BlocksHash, VersionHash: f.VersionHash})
 }
 
-func (db *Lowlevel) recordIndirectionHashes(hs IndirectionHashesOnly) {
+func (db *Lowlevel) recordIndirectionHashes(hs *dbproto.IndirectionHashesOnly) {
 	// must be called with gcMut held (at least read-held)
 	if db.blockFilter != nil && len(hs.BlocksHash) > 0 {
 		db.blockFilter.add(hs.BlocksHash)
@@ -974,7 +969,7 @@ func (b *bloomFilter) hash(id []byte) uint64 {
 	}
 	var h maphash.Hash
 	h.SetSeed(b.seed)
-	h.Write(id)
+	_, _ = h.Write(id)
 	return h.Sum64()
 }
 
@@ -1043,7 +1038,7 @@ func (db *Lowlevel) getMetaAndCheckGCLocked(folder string) (*metadataTracker, er
 func (db *Lowlevel) loadMetadataTracker(folder string) (*metadataTracker, error) {
 	meta := newMetadataTracker(db.keyer, db.evLogger)
 	if err := meta.fromDB(db, []byte(folder)); err != nil {
-		if err == errMetaInconsistent {
+		if errors.Is(err, errMetaInconsistent) {
 			l.Infof("Stored folder metadata for %q is inconsistent; recalculating", folder)
 		} else {
 			l.Infof("No stored folder metadata for %q; recalculating", folder)
@@ -1079,7 +1074,7 @@ func (db *Lowlevel) recalcMeta(folderStr string) (*metadataTracker, error) {
 	defer t.close()
 
 	var deviceID protocol.DeviceID
-	err = t.withAllFolderTruncated(folder, func(device []byte, f FileInfoTruncated) bool {
+	err = t.withAllFolderTruncated(folder, func(device []byte, f protocol.FileInfo) bool {
 		copy(deviceID[:], device)
 		meta.addFile(deviceID, f)
 		return true
@@ -1088,7 +1083,7 @@ func (db *Lowlevel) recalcMeta(folderStr string) (*metadataTracker, error) {
 		return nil, err
 	}
 
-	err = t.withGlobal(folder, nil, true, func(f protocol.FileIntf) bool {
+	err = t.withGlobal(folder, nil, true, func(f protocol.FileInfo) bool {
 		meta.addFile(protocol.GlobalDeviceID, f)
 		return true
 	})
@@ -1097,7 +1092,7 @@ func (db *Lowlevel) recalcMeta(folderStr string) (*metadataTracker, error) {
 	}
 
 	meta.emptyNeeded(protocol.LocalDeviceID)
-	err = t.withNeed(folder, protocol.LocalDeviceID[:], true, func(f protocol.FileIntf) bool {
+	err = t.withNeed(folder, protocol.LocalDeviceID[:], true, func(f protocol.FileInfo) bool {
 		meta.addNeeded(protocol.LocalDeviceID, f)
 		return true
 	})
@@ -1106,7 +1101,7 @@ func (db *Lowlevel) recalcMeta(folderStr string) (*metadataTracker, error) {
 	}
 	for _, device := range meta.devices() {
 		meta.emptyNeeded(device)
-		err = t.withNeed(folder, device[:], true, func(f protocol.FileIntf) bool {
+		err = t.withNeed(folder, device[:], true, func(f protocol.FileInfo) bool {
 			meta.addNeeded(device, f)
 			return true
 		})
@@ -1140,7 +1135,7 @@ func (db *Lowlevel) verifyLocalSequence(curSeq int64, folder string) (bool, erro
 		return false, err
 	}
 	ok := true
-	if err := t.withHaveSequence([]byte(folder), curSeq+1, func(fi protocol.FileIntf) bool {
+	if err := t.withHaveSequence([]byte(folder), curSeq+1, func(_ protocol.FileInfo) bool {
 		ok = false // we got something, which we should not have
 		return false
 	}); err != nil {
@@ -1212,8 +1207,7 @@ func (db *Lowlevel) repairSequenceGCLocked(folderStr string, meta *metadataTrack
 			}
 			return 0, err
 		}
-		fi := intf.(protocol.FileInfo)
-		if sk, err = t.keyer.GenerateSequenceKey(sk, folder, fi.Sequence); err != nil {
+		if sk, err = t.keyer.GenerateSequenceKey(sk, folder, intf.Sequence); err != nil {
 			return 0, err
 		}
 		switch dk, err = t.Get(sk); {
@@ -1224,14 +1218,14 @@ func (db *Lowlevel) repairSequenceGCLocked(folderStr string, meta *metadataTrack
 			fallthrough
 		case !bytes.Equal(it.Key(), dk):
 			fixed++
-			fi.Sequence = meta.nextLocalSeq()
-			if sk, err = t.keyer.GenerateSequenceKey(sk, folder, fi.Sequence); err != nil {
+			intf.Sequence = meta.nextLocalSeq()
+			if sk, err = t.keyer.GenerateSequenceKey(sk, folder, intf.Sequence); err != nil {
 				return 0, err
 			}
 			if err := t.Put(sk, it.Key()); err != nil {
 				return 0, err
 			}
-			if err := t.putFile(it.Key(), fi); err != nil {
+			if err := t.putFile(it.Key(), intf); err != nil {
 				return 0, err
 			}
 		}
@@ -1315,9 +1309,8 @@ func (db *Lowlevel) checkLocalNeed(folder []byte) (int, error) {
 		}
 	}
 	next()
-	t.withNeedIteratingGlobal(folder, protocol.LocalDeviceID[:], true, func(fi protocol.FileIntf) bool {
-		f := fi.(FileInfoTruncated)
-		for !needDone && needName < f.Name {
+	itErr := t.withNeedIteratingGlobal(folder, protocol.LocalDeviceID[:], true, func(fi protocol.FileInfo) bool {
+		for !needDone && needName < fi.Name {
 			repaired++
 			if err = t.Delete(dbi.Key()); err != nil && !backend.IsNotFound(err) {
 				return false
@@ -1325,24 +1318,27 @@ func (db *Lowlevel) checkLocalNeed(folder []byte) (int, error) {
 			l.Debugln("check local need: removing", needName)
 			next()
 		}
-		if needName == f.Name {
+		if needName == fi.Name {
 			next()
 		} else {
 			repaired++
-			key, err = t.keyer.GenerateNeedFileKey(key, folder, []byte(f.Name))
+			key, err = t.keyer.GenerateNeedFileKey(key, folder, []byte(fi.Name))
 			if err != nil {
 				return false
 			}
 			if err = t.Put(key, nil); err != nil {
 				return false
 			}
-			l.Debugln("check local need: adding", f.Name)
+			l.Debugln("check local need: adding", fi.Name)
 		}
 		return true
 	})
 	if err != nil {
 		return 0, err
 	}
+	if itErr != nil {
+		return 0, itErr
+	}
 
 	for !needDone {
 		repaired++
@@ -1443,13 +1439,6 @@ func (db *Lowlevel) checkErrorForRepair(err error) {
 	}
 }
 
-// unchanged checks if two files are the same and thus don't need to be updated.
-// Local flags or the invalid bit might change without the version
-// being bumped.
-func unchanged(nf, ef protocol.FileIntf) bool {
-	return ef.FileVersion().Equal(nf.FileVersion()) && ef.IsInvalid() == nf.IsInvalid() && ef.FileLocalFlags() == nf.FileLocalFlags()
-}
-
 func (db *Lowlevel) handleFailure(err error) {
 	db.checkErrorForRepair(err)
 	if shouldReportFailure(err) {
