@@ -10,6 +10,7 @@ import (
 	"os"
 	"path/filepath"
 	"strings"
+	"time"
 )
 
 var execExts map[string]bool
@@ -37,15 +38,15 @@ func (e basicFileInfo) Mode() FileMode {
 		// NTFS deduped files. Remove the symlink bit.
 		m &^= os.ModeSymlink
 	}
-	// Set executable bits on files with executable extenions (.exe, .bat, etc).
+	// Set executable bits on files with executable extensions (.exe, .bat, etc).
 	if isWindowsExecutable(e.Name()) {
-		m |= 0111
+		m |= 0o111
 	}
 	// There is no user/group/others in Windows' read-only attribute, and
 	// all "w" bits are set if the file is not read-only.  Do not send these
 	// group/others-writable bits to other devices in order to avoid
 	// unexpected world-writable files on other platforms.
-	m &^= 0022
+	m &^= 0o022
 	return FileMode(m)
 }
 
@@ -57,6 +58,10 @@ func (e basicFileInfo) Group() int {
 	return -1
 }
 
+func (basicFileInfo) InodeChangeTime() time.Time {
+	return time.Time{}
+}
+
 // osFileInfo converts e to os.FileInfo that is suitable
 // to be passed to os.SameFile.
 func (e *basicFileInfo) osFileInfo() os.FileInfo {
