@@ -12,6 +12,7 @@ import (
 	"io"
 	"log"
 	"os"
+	"slices"
 	"strings"
 	"sync"
 	"time"
@@ -31,7 +32,7 @@ const (
 )
 
 const (
-	DefaultFlags = log.Ltime
+	DefaultFlags = log.Ltime | log.Ldate
 	DebugFlags   = log.Ltime | log.Ldate | log.Lmicroseconds | log.Lshortfile
 )
 
@@ -52,7 +53,6 @@ type Logger interface {
 	Warnf(format string, vals ...interface{})
 	ShouldDebug(facility string) bool
 	SetDebug(facility string, enabled bool)
-	IsTraced(facility string) bool
 	Facilities() map[string]string
 	FacilityDebugging() []string
 	NewFacility(facility, description string) Logger
@@ -63,7 +63,7 @@ type logger struct {
 	handlers   [NumLevels][]MessageHandler
 	facilities map[string]string   // facility name => description
 	debug      map[string]struct{} // only facility names with debugging enabled
-	traces     string
+	traces     []string
 	mut        sync.Mutex
 }
 
@@ -80,9 +80,21 @@ func New() Logger {
 }
 
 func newLogger(w io.Writer) Logger {
+	traces := strings.FieldsFunc(os.Getenv("STTRACE"), func(r rune) bool {
+		return strings.ContainsRune(",; ", r)
+	})
+
+	if len(traces) > 0 {
+		if slices.Contains(traces, "all") {
+			traces = []string{"all"}
+		} else {
+			slices.Sort(traces)
+		}
+	}
+
 	return &logger{
 		logger:     log.New(w, "", DefaultFlags),
-		traces:     os.Getenv("STTRACE"),
+		traces:     traces,
 		facilities: make(map[string]string),
 		debug:      make(map[string]struct{}),
 	}
@@ -118,6 +130,7 @@ func (l *logger) callHandlers(level LogLevel, s string) {
 func (l *logger) Debugln(vals ...interface{}) {
 	l.debugln(3, vals...)
 }
+
 func (l *logger) debugln(level int, vals ...interface{}) {
 	s := fmt.Sprintln(vals...)
 	l.mut.Lock()
@@ -130,6 +143,7 @@ func (l *logger) debugln(level int, vals ...interface{}) {
 func (l *logger) Debugf(format string, vals ...interface{}) {
 	l.debugf(3, format, vals...)
 }
+
 func (l *logger) debugf(level int, format string, vals ...interface{}) {
 	s := fmt.Sprintf(format, vals...)
 	l.mut.Lock()
@@ -215,9 +229,18 @@ func (l *logger) SetDebug(facility string, enabled bool) {
 	}
 }
 
-// IsTraced returns whether the facility name is contained in STTRACE.
-func (l *logger) IsTraced(facility string) bool {
-	return strings.Contains(l.traces, facility) || l.traces == "all"
+// isTraced returns whether the facility name is contained in STTRACE.
+func (l *logger) isTraced(facility string) bool {
+	if len(l.traces) > 0 {
+		if l.traces[0] == "all" {
+			return true
+		}
+
+		_, found := slices.BinarySearch(l.traces, facility)
+		return found
+	}
+
+	return false
 }
 
 // FacilityDebugging returns the set of facilities that have debugging
@@ -246,7 +269,7 @@ func (l *logger) Facilities() map[string]string {
 
 // NewFacility returns a new logger bound to the named facility.
 func (l *logger) NewFacility(facility, description string) Logger {
-	l.SetDebug(facility, l.IsTraced(facility))
+	l.SetDebug(facility, l.isTraced(facility))
 
 	l.mut.Lock()
 	l.facilities[facility] = description
