@@ -15,7 +15,7 @@ import (
 	"github.com/syncthing/syncthing/lib/events"
 	"github.com/syncthing/syncthing/lib/ignore"
 	"github.com/syncthing/syncthing/lib/protocol"
-	"github.com/syncthing/syncthing/lib/util"
+	"github.com/syncthing/syncthing/lib/semaphore"
 	"github.com/syncthing/syncthing/lib/versioner"
 )
 
@@ -28,27 +28,27 @@ receiveOnlyFolder is a folder that does not propagate local changes outward.
 It does this by the following general mechanism (not all of which is
 implemented in this file):
 
-- Local changes are scanned and versioned as usual, but get the
-  FlagLocalReceiveOnly bit set.
+  - Local changes are scanned and versioned as usual, but get the
+    FlagLocalReceiveOnly bit set.
 
-- When changes are sent to the cluster this bit gets converted to the
-  Invalid bit (like all other local flags, currently) and also the Version
-  gets set to the empty version. The reason for clearing the Version is to
-  ensure that other devices will not consider themselves out of date due to
-  our change.
+  - When changes are sent to the cluster this bit gets converted to the
+    Invalid bit (like all other local flags, currently) and also the Version
+    gets set to the empty version. The reason for clearing the Version is to
+    ensure that other devices will not consider themselves out of date due to
+    our change.
 
-- The database layer accounts sizes per flag bit, so we can know how many
-  files have been changed locally. We use this to trigger a "Revert" option
-  on the folder when the amount of locally changed data is nonzero.
+  - The database layer accounts sizes per flag bit, so we can know how many
+    files have been changed locally. We use this to trigger a "Revert" option
+    on the folder when the amount of locally changed data is nonzero.
 
-- To revert we take the files which have changed and reset their version
-  counter down to zero. The next pull will replace our changed version with
-  the globally latest. As this is a user-initiated operation we do not cause
-  conflict copies when reverting.
+  - To revert we take the files which have changed and reset their version
+    counter down to zero. The next pull will replace our changed version with
+    the globally latest. As this is a user-initiated operation we do not cause
+    conflict copies when reverting.
 
-- When pulling normally (i.e., not in the revert case) with local changes,
-  normal conflict resolution will apply. Conflict copies will be created,
-  but not propagated outwards (because receive only, right).
+  - When pulling normally (i.e., not in the revert case) with local changes,
+    normal conflict resolution will apply. Conflict copies will be created,
+    but not propagated outwards (because receive only, right).
 
 Implementation wise a receiveOnlyFolder is just a sendReceiveFolder that
 sets an extra bit on local changes and has a Revert method.
@@ -57,7 +57,7 @@ type receiveOnlyFolder struct {
 	*sendReceiveFolder
 }
 
-func newReceiveOnlyFolder(model *model, fset *db.FileSet, ignores *ignore.Matcher, cfg config.FolderConfiguration, ver versioner.Versioner, evLogger events.Logger, ioLimiter *util.Semaphore) service {
+func newReceiveOnlyFolder(model *model, fset *db.FileSet, ignores *ignore.Matcher, cfg config.FolderConfiguration, ver versioner.Versioner, evLogger events.Logger, ioLimiter *semaphore.Semaphore) service {
 	sr := newSendReceiveFolder(model, fset, ignores, cfg, ver, evLogger, ioLimiter).(*sendReceiveFolder)
 	sr.localFlags = protocol.FlagLocalReceiveOnly // gets propagated to the scanner, and set on locally changed files
 	return &receiveOnlyFolder{sr}
@@ -92,8 +92,7 @@ func (f *receiveOnlyFolder) revert() error {
 		return err
 	}
 	defer snap.Release()
-	snap.WithHave(protocol.LocalDeviceID, func(intf protocol.FileIntf) bool {
-		fi := intf.(protocol.FileInfo)
+	snap.WithHave(protocol.LocalDeviceID, func(fi protocol.FileInfo) bool {
 		if !fi.IsReceiveOnlyChanged() {
 			// We're only interested in files that have changed locally in
 			// receive only mode.
@@ -126,7 +125,12 @@ func (f *receiveOnlyFolder) revert() error {
 			}
 			fi.SetDeleted(f.shortID)
 			fi.Version = protocol.Vector{} // if this file ever resurfaces anywhere we want our delete to be strictly older
-		case gf.IsEquivalentOptional(fi, f.modTimeWindow, false, false, protocol.FlagLocalReceiveOnly):
+		case gf.IsEquivalentOptional(fi, protocol.FileInfoComparison{
+			ModTimeWindow:   f.modTimeWindow,
+			IgnoreFlags:     protocol.FlagLocalReceiveOnly,
+			IgnoreOwnership: !f.SyncOwnership,
+			IgnoreXattrs:    !f.SyncXattrs,
+		}):
 			// What we have locally is equivalent to the global file.
 			fi = gf
 		default:
@@ -211,7 +215,7 @@ func (q *deleteQueue) flush(snap *db.Snapshot) ([]string, error) {
 	for _, dir := range q.dirs {
 		if err := q.handler.deleteDirOnDisk(dir, snap, q.scanChan); err == nil {
 			deleted = append(deleted, dir)
-		} else if err != nil && firstError == nil {
+		} else if firstError == nil {
 			firstError = err
 		}
 	}
