@@ -12,7 +12,7 @@ import (
 	"github.com/syncthing/syncthing/lib/events"
 	"github.com/syncthing/syncthing/lib/ignore"
 	"github.com/syncthing/syncthing/lib/protocol"
-	"github.com/syncthing/syncthing/lib/util"
+	"github.com/syncthing/syncthing/lib/semaphore"
 	"github.com/syncthing/syncthing/lib/versioner"
 )
 
@@ -24,7 +24,7 @@ type sendOnlyFolder struct {
 	folder
 }
 
-func newSendOnlyFolder(model *model, fset *db.FileSet, ignores *ignore.Matcher, cfg config.FolderConfiguration, _ versioner.Versioner, evLogger events.Logger, ioLimiter *util.Semaphore) service {
+func newSendOnlyFolder(model *model, fset *db.FileSet, ignores *ignore.Matcher, cfg config.FolderConfiguration, _ versioner.Versioner, evLogger events.Logger, ioLimiter *semaphore.Semaphore) service {
 	f := &sendOnlyFolder{
 		folder: newFolder(model, fset, ignores, cfg, evLogger, ioLimiter, nil),
 	}
@@ -32,7 +32,7 @@ func newSendOnlyFolder(model *model, fset *db.FileSet, ignores *ignore.Matcher,
 	return f
 }
 
-func (f *sendOnlyFolder) PullErrors() []FileError {
+func (*sendOnlyFolder) PullErrors() []FileError {
 	return nil
 }
 
@@ -48,31 +48,34 @@ func (f *sendOnlyFolder) pull() (bool, error) {
 		return false, err
 	}
 	defer snap.Release()
-	snap.WithNeed(protocol.LocalDeviceID, func(intf protocol.FileIntf) bool {
+	snap.WithNeed(protocol.LocalDeviceID, func(file protocol.FileInfo) bool {
 		batch.FlushIfFull()
 
-		file := intf.(protocol.FileInfo)
-
-		if f.ignores.ShouldIgnore(intf.FileName()) {
+		if f.ignores.Match(file.FileName()).IsIgnored() {
 			file.SetIgnored()
 			batch.Append(file)
 			l.Debugln(f, "Handling ignored file", file)
 			return true
 		}
 
-		curFile, ok := snap.Get(protocol.LocalDeviceID, intf.FileName())
+		curFile, ok := snap.Get(protocol.LocalDeviceID, file.FileName())
 		if !ok {
-			if intf.IsInvalid() {
+			if file.IsInvalid() {
 				// Global invalid file just exists for need accounting
 				batch.Append(file)
-			} else if intf.IsDeleted() {
+			} else if file.IsDeleted() {
 				l.Debugln("Should never get a deleted file as needed when we don't have it")
 				f.evLogger.Log(events.Failure, "got deleted file that doesn't exist locally as needed when pulling on send-only")
 			}
 			return true
 		}
 
-		if !file.IsEquivalentOptional(curFile, f.modTimeWindow, f.IgnorePerms, false, 0) {
+		if !file.IsEquivalentOptional(curFile, protocol.FileInfoComparison{
+			ModTimeWindow:   f.modTimeWindow,
+			IgnorePerms:     f.IgnorePerms,
+			IgnoreOwnership: !f.SyncOwnership,
+			IgnoreXattrs:    !f.SyncXattrs,
+		}) {
 			return true
 		}
 
@@ -106,8 +109,7 @@ func (f *sendOnlyFolder) override() error {
 		return err
 	}
 	defer snap.Release()
-	snap.WithNeed(protocol.LocalDeviceID, func(fi protocol.FileIntf) bool {
-		need := fi.(protocol.FileInfo)
+	snap.WithNeed(protocol.LocalDeviceID, func(need protocol.FileInfo) bool {
 		_ = batch.FlushIfFull()
 
 		have, ok := snap.Get(protocol.LocalDeviceID, need.Name)
