@@ -9,12 +9,12 @@ package scanner
 import (
 	"bytes"
 	"context"
+	"crypto/sha256"
 	"hash"
 	"hash/adler32"
 	"io"
 
 	"github.com/syncthing/syncthing/lib/protocol"
-	"github.com/syncthing/syncthing/lib/sha256"
 )
 
 var SHA256OfNothing = []uint8{0xe3, 0xb0, 0xc4, 0x42, 0x98, 0xfc, 0x1c, 0x14, 0x9a, 0xfb, 0xf4, 0xc8, 0x99, 0x6f, 0xb9, 0x24, 0x27, 0xae, 0x41, 0xe4, 0x64, 0x9b, 0x93, 0x4c, 0xa4, 0x95, 0x99, 0x1b, 0x78, 0x52, 0xb8, 0x55}
@@ -139,4 +139,4 @@ func (noopHash) Write([]byte) (int, error) { return 0, nil }
 
 type noopCounter struct{}
 
-func (c *noopCounter) Update(bytes int64) {}
+func (*noopCounter) Update(_ int64) {}
